var Toolbox = function()
{

//--------------------------------------------------------------------//	

	this.imageToSurface = function( imgImage )
	{
		if(typeof imgImage.width == 'number')
		{
			var s = new Surface('0', imgImage.width, imgImage.height );
			s.drawImage( imgImage, 0, 0 );
			return s;
		} else {
			return false;
		}
	};
	
//--------------------------------------------------------------------//	

	this.RunPrefixMethod = function( obj, method ) 
	{
		var prefixMethod = ["webkit", "moz", "ms", "o", ""];
		var p = 0, m, t;
		while (p < prefixMethod.length && !obj[m]) {
			m = method;
			if (prefixMethod[p] == "") {
				m = m.substr(0,1).toLowerCase() + m.substr(1);
			}
			m = prefixMethod[p] + m;
			t = typeof obj[m];
			if (t != "undefined") {
				prefixMethod = [prefixMethod[p]];
				if ( t == "function" ) obj[m]();
				document.getElementById('canvas').mozRequestFullScreen();
				return true;
			}
			p++;
		}
	}

//--------------------------------------------------------------------//	

	this.setFullScreen = function( idCanvas, status )
	{
		var e = document.getElementById( idCanvas );
		if( status == 0 )
		{
			this.RunPrefixMethod( document, "CancelFullScreen" );
		} else {
			var r = this.RunPrefixMethod( e, "RequestFullScreen" );
		}
	}

//--------------------------------------------------------------------//	

	this.FrameTimer = function() 
	{
		_lastTick     = (new Date()).getTime();
		_frameSpacing = 0;
		
		this.getSeconds = function() 
		{
			var seconds = _frameSpacing / 1000;
			if( isNaN( seconds ) ) 
			{
				return 0;
			}
			return seconds;
		};
	 
		this.tick = function() 
		{
			var currentTick    = ( new Date() ).getTime();
			_frameSpacing = currentTick - _lastTick;
			_lastTick     = currentTick;
		}

	};
 
 //--------------------------------------------------------------------//	
 
};
/*
	Clase: surface.class.js
	Este objeto debería definir una superficie
	si se le pasa un canvas como parametro, usa ese canvas como superficie
	si se le pasa un id de un objeto canvas lo instancia como superficie
	si se le pasa un '0' como cadena y un ancho y alto crea un nuevo objeto canvas y lo usa como superficie
	ej:
	Para crear una superficie de un objeto 'canvas' existente en el DOM pasamos el id del canvas
	var s = new Surface( 'superficie' );
	
	Para crear una superficie partiendo de un canvas existente, pasamos el canvas como parametro
	var s = new Surface( objCanvas );
	
	Para crear una nueva superficie de 800x600
	var s = new Surface( '0', 800, 600 );
	
*/ 

var Surface = function( strSurface, strWidth, strHeight )
{
	var _canSurface;
	var ctxSurface;
	
	
////////////////////////////////////////////////////////////////////////
//           INICIALIZA EL CANVAS Y EL CONTEXTO                       //
////////////////////////////////////////////////////////////////////////	
	this._init = function( strSurface, strWidth, strHeight )
	{
		if(typeof strSurface == 'canvas' )
		{
			_canSurface = strSurface;
		} else {
			if( strSurface != '0')
			{
				_canSurface  = document.getElementById( strSurface );
				if( strWidth  ) _canSurface.width  = strWidth;
				if( strHeight ) _canSurface.height = strHeight;
			} else {
				_canSurface        = document.createElement( 'canvas' );
				_canSurface.width  = strWidth;
				_canSurface.height = strHeight;
			}
		}
		this.ctxSurface = _canSurface.getContext( "2d" );
	}
	
	
	this.clearAll = function()
	{
		this.ctxSurface.clearRect(0, 0, _canSurface.width, _canSurface.height );
	}
	
	this.clearRect = function( intX, intY, intWidth, intHeight )
	{
		this.ctxSurface.clearRect( intX, intY, intWidth, intHeight );
	}
	
// context.drawImage(img,sx,sy,swidth,sheight,x,y,width,height);	
	this.drawImageRect = function( canOrigin, intOrigLeft, intOrigTop, intOrigWidth, intOrigHeight, intX, intY )
	{
		this.ctxSurface.drawImage(  canOrigin, 
									intOrigLeft, 
									intOrigTop, 
									intOrigWidth, 
									intOrigHeight, 
									intX, 
									intY,
									intOrigWidth, 
									intOrigHeight );
	}
	
	this.drawImage = function( canOrigin, intX, intY )
	{
		this.ctxSurface.drawImage( canOrigin, intX, intY );
	}
	
	this.getCanvas = function()
	{
		return _canSurface;
	}

	this.Size = function( set )
	{
		if( set )
		{
			if( set.width  ) _canSurface.width  = set.width;
			if( set.height ) _canSurface.height = set.height;
		} else {
			return { width : _canSurface.width, height : _canSurface.height }
		}
	}
	
////////////////////////////////////////////////////////////////////////
//                     INICIALIZA LA CLASE                            //
////////////////////////////////////////////////////////////////////////
	if( ! strSurface && ! strWidth && ! strHeight ) 
	{
		console.log('new Suerface(): ERROR - Faltan parámetros de inicialización');
	} else { 
		this._init( strSurface, strWidth, strHeight );
	}
}
	
var Primitives = Surface;

////////////////////////////////////////////////////////////////////////////////////////////////
//                              RUTINAS DE PROCESO DE TEXTOS                                  //
////////////////////////////////////////////////////////////////////////////////////////////////

Primitives.prototype = 
{
	// Prototipo que dibuja un texto en pantalla
	_text : function(font, color, x, y, text, bold, outline)
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.font        = font;
		if( outline == false ) this.ctxSurface.fillText( text, x, y );
		if( bold    == true  ) this.ctxSurface.strokeText( text, x, y );
	},

	// Pone un texto normal o con estilo negrita (por font)
	text : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, false, false );
	},
	
	// pone un texto y dibuja un outline para hacerlo negrita (por canvas)
	// si se pasa bold en el font entonces tenes hipernegrita
	text_bold : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, true, false);
	},

	// Pone un texto en outline
	text_outline : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, true, true );
	},

	// Pone un texto normal o con estilo negrita (por font)
	textSize : function( font, text )
	{
		var pa  = pa || document.body;
		var who = document.createElement( 'div' );
		
		who.style.visibility = 'hidden';
		who.style.position   = 'absolute';
		who.style.font       = font;
		who.appendChild( document.createTextNode( text ) );
		
		pa.appendChild( who );
		var fs = [ who.offsetWidth, who.offsetHeight ];
		pa.removeChild( who );
		return fs;
	},
	
////////////////////////////////////////////////////////////////////////////////////////////////
//                                RUTINAS DE RENDER EN CANVAS                                 //
////////////////////////////////////////////////////////////////////////////////////////////////

	// Funcion que devuelve un gradiente lineal de 2 pasos
	gradient : function( x1, y1, x2, y2, color1, color2 )
	{
		var gradient = this.ctxSurface.createLinearGradient( x1, y1, x2, y2 );
		gradient.addColorStop(0, color1);
		gradient.addColorStop(1, color2);
		return gradient;
	},
		
	// Dibuja una linea del color especificado entre los puntos pasados x parametro
	line : function( x1, y1, x2, y2, color )
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.beginPath();
		this.ctxSurface.moveTo( x1, y1 );
		this.ctxSurface.lineTo( x2, y2 );
		this.ctxSurface.stroke();
	},

	// Dibuja una linea con un gradiente
	line_gradient : function( x1, y1, x2, y2, color1, color2 )
	{
		var grad = this.gradient( x1, y1, x2, y2, color1, color2 );
		this.line(x1, y1, x2, y2, grad );
	},

	// Funcion que dibuja un rectangulo del tamaño especificado en el canvas
	// El color puede ser reemplazado por un gradiente
	line_fill : function( x, y, width, height, color ) 
	{
		this.ctxSurface.fillStyle = color;
		this.ctxSurface.fillRect( x, y, width, height );
	},
	
	// Funcion que dibuja un punto del tamaño especificado en el canvas
	pset : function( x, y, side, color ) 
	{
		this.ctxSurface.fillStyle = color;
		this.ctxSurface.fillRect( x, y, side, side );
	},

	// Prototipo de arco que permite dibujar arcos (abiertos o cerrado) y circulos 
	_arc : function( x, y, radio, degini, degfin, color, closed, fill)
	{
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.beginPath();
		radini = this.deg2rad( degini );
		radfin = this.deg2rad( degfin );
		this.ctxSurface.arc( x, y, radio, radini, radfin, true ); 
		if( closed == true ) this.ctxSurface.closePath();
		if( fill   == true ) this.ctxSurface.fill();
		this.ctxSurface.stroke();
	},
	
	circle : function(x, y, radio, color, fill )
	{
		if( ! fill ) fill = false;
		this._arc( x, y, radio, 0, 360, color, true, fill );
	},
	
	// Arco abierto y hueco
	arc : function(x, y, radio, degini, degfin, color )
	{
		this._arc(x, y, radio, degini, degfin, color, false, false );
	},
	
	// Arco cerrado y hueco
	arc_close : function( x, y, radio, degini, degfin, color )
	{
		this._arc( x, y, radio, degini, degfin, color, true, false );
	},
		
	// Arco cerrado y relleno
	arc_fill : function( x, y, radio, degini, degfin, color )
	{
		this._arc( x, y, radio, degini, degfin, color, true, true );
	},
	
	// Rutina de dibujo de poligonos irregulares
	// Esta rutina es el prototipo que permite poligonos
	// abiertos, cerrados, rellenos y con gradientes
	_poly : function( vertex, color, closed, fill )
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.beginPath();
		var start = true;
		for ( var v in vertex ) {
			var k = vertex[v].split(',');
			if(start == true){
				this.ctxSurface.moveTo( k[ 0 ], k[ 1 ] );
			} else {
				this.ctxSurface.lineTo( k[ 0 ], k[ 1 ] );
			}
			start = false;
		}
		if( closed == true ) this.ctxSurface.closePath();
		if( fill   == true ) this.ctxSurface.fill();
		ctx.stroke();
	},
	
	// Acceso simple para poligono abierto
	poly : function( vertex, color )
	{
		this._poly( vertex, color, false, false );
	},
	
	// Acceso simple para poligono cerrado sin relleno
	poly_closed : function(vertex, color )
	{
		this._poly( vertex, color, true, false );
	},
	
	// Acceso simple para poligono 
	poly_fill : function(vertex, color )
	{
		this._poly( vertex, color, true, true );
	},
	
////////////////////////////////////////////////////////////////////////////////////////////////
//                             RUTINAS AUXILIARES INDISPENSABLES                              //
////////////////////////////////////////////////////////////////////////////////////////////////

	// Funcion que pasa de grados sexagecimales a radianes
	deg2rad : function( deg ) 
	{
		return ( deg * Math.PI ) / 180;
	}

};
/*
	Class: layer_tiles.class.js
	
*/ 


var Layer_Tiles = function()
{
	var _type  = "tiles";
	var _name  = "";
	var _layer = new Surface('0',1104, 672);
	var _tilemap;
	var _viewport = new Surface('0', 1008, 576 );;
	var _levelMap;
	
	var _collitionMask;

	var _intTilemap;
	
/* ------------------------------------------------------------------ */	

	this.initialize = function( data )
	{
		if( data.tilemap      ) _tilemap       = data.tilemap;
		if( data.levelmap     ) _levelMap      = data.levelmap;
		if( data.collitionmap ) _collitionMask = this.expandCollitionMap( data.collitionmap );

		// Define a 23x14 tiles viewport (tiles of 48x48 pixels)
		// 1104 = 23*48
		// 672 = 14*48
		_intTilemap = (data.tilemap.Size().width / 48  );
	}

/* ------------------------------------------------------------------ */	

	this.expandCollitionMap = function( arrColMap )
	{		
		_collitionMask = new Array( arrColMap.length );
		for(var i=0; i < arrColMap.length; i++)
		{
			_collitionMask[ i ] = new Array( 6 );
			for(var j=0; j<6; j++)
			{
				_collitionMask[ i ][ j ] = new Array( 6 );
				var z = parseInt( arrColMap[ i ].substr( ( j * 2 ), 2 ), 16 );
				
				var x = 32;
				for(var k = 0; k < 6; k++ )
				{
					_collitionMask[ i ][ j ][ k ] = ((z & x) > 0 ? 1 : 0 );
					x = x >> 1;
				}
			}
		}
	}
	
/* ------------------------------------------------------------------ */	

	this.checkCollition = function( coords )
	{
		for( var i = 0; i < coords.length; i++ )
		{
			var tileX    = Math.floor( coords[ i ][ 0 ] / 48 );
			var tileY    = Math.floor( coords[ i ][ 1 ] / 48 );
			var tilePosX = Math.floor( ( coords[ i ][ 0 ] % 48 ) / 8 );
			var tilePosY = Math.floor( ( coords[ i ][ 1 ] % 48 ) / 8 );
			var tileId   = _levelMap[ tileX ][ tileY ];
			tileId--;
			if( tileId >= 0 ) 
			{
				if( _collitionMask[ tileId ][ tilePosY ][ tilePosX ] == 1) return true;
			}
		}
		return false;
	}

/* ------------------------------------------------------------------ */	
	
	this.getTileTop = function( intTileNum )
	{
		return Math.floor( ( intTileNum - 1 ) / _intTilemap ) * 48;
	}

/* ------------------------------------------------------------------ */	
	
	this.getTileLeft = function( intTileNum )
	{
		return ( ( intTileNum - 1 ) % _intTilemap ) * 48;
	}
	
/* ------------------------------------------------------------------ */	

	this.fillTiles = function( intX, intY )
	{
		_layer.clearAll();
		for( var i = intX; i < ( intX + 23 ); i++ )
			for( var j = intY; j < ( intY + 14 ); j++ )
			{
				if( _levelMap[ i ][ j ] > 0 )
				{
					_layer.drawImageRect(
						_tilemap.getCanvas(), 
						this.getTileLeft( _levelMap[ i ][ j ] ), 
						this.getTileTop( _levelMap[ i ][ j ] ), 
						48, 48, ( 48 * ( i - intX ) ), ( 48 * ( j - intY ) )
					);
				}
			}
	}
	
/* ------------------------------------------------------------------ */	

	this.getCanvas = function()
	{
		return _layer.getCanvas();
	}
	
/* ------------------------------------------------------------------ */	

	this.getViewport = function( intX, intY)
	{
		// Sanity check para mantener el viewport dentro del mundo
		intX = ( intX < 0 ? 0 : ( intX > 2112 ? 2112 : intX ) );
		intY = ( intY < 0 ? 0 : ( intY > 384 ? 384 : intY ) );
		
		var intTileX = Math.floor( intX / 48 );
		var intTileY = Math.floor( intY / 48 );
		
		// en principio vamos a hacer un full redraw de cada cuadro
		// una vez que esto ande veré de optimizar esto par no desperdiciar tantos recursos :-)
		this.fillTiles( intTileX, intTileY );
		
		intTileX = intX % 48;
		intTileY = intY % 48;
		_viewport.drawImageRect( _layer.getCanvas(), intTileX, intTileY, 1008, 576, 0, 0 );
		
		return _viewport;
	}
	
}
var Sprite = function()
{
	
//--------------------------------------------------------------------//
	
	var _Position  = { x: 0, y : 0 };
	var _SpriteTiles;
	var _Frameset  = [];         // Collection of frames format [ { Pic : <Surface>, time : <seconds> } ]
	var _Animation = [];         // Collection of animations format { 'name' : [0,1,2...n], 'name2' : [...] } the array  is a collection of indexes of the _Frameset object
	var _CurAnim   = '';         // Name of the current animation
	var _FrameIdx  = 0;          // Index of the frame in the current animation
	var _AnimLoop  = true;
	var _Width     = 0;
	var _Tic       = 0;
	var _Name      = "Sprite";   // Name of the current sprite
	var _EOA       = false;     // End of Animation

//--------------------------------------------------------------------//

	this.name = function( set )
	{
		if( set )
		{
			_Name = set;
		} else {
			return _Name;
		}
	}
	
//--------------------------------------------------------------------//	

	this.position = function( set )
	{
		if( set )
		{
			if( set.x ) _Position.x = set.x;
			if( set.y ) _Position.y = set.y;
		} else {
			return _Position;
		}
	}

//--------------------------------------------------------------------//
	
	this.eoa = function()
	{
		return _EOA;
	}
	
//--------------------------------------------------------------------//	

	this.tic = function( set )
	{
		if( set )
		{
			_Tic -= set;
			if( _Tic <= 0)
			{
				_FrameIdx++;
				if( _Animation[ _CurAnim ].length == _FrameIdx )
				{
					if( _AnimLoop == false )
					{
						_EOA = true;
						_FrameIdx--;
					} else {
						_FrameIdx = 0;
						_Tic = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
					}
				} else {
					_Tic = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
				}
			}
		} 
	}
	
//--------------------------------------------------------------------//

	this.getFrame = function()
	{
		return _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Pic.getCanvas();
	}
	
//--------------------------------------------------------------------//
	
	this.getTilePos = function( intTileNum, intTileWidth )
	{
		return { 
					x : ( intTileNum * _Width ) % intTileWidth, 
					y : Math.floor( (intTileNum * _Width) / intTileWidth ) * _Width
				};
	}

//--------------------------------------------------------------------//
	
	this.loadSpriteTiles = function( spriteTiles, spriteWidth )
	{
		_SpriteTiles = spriteTiles;
		_Width       = spriteWidth;
	}
	
//--------------------------------------------------------------------//

	this.loadFrames = function( sequence )
	{
		var tmpWidth = _SpriteTiles.Size().width;
		for( var i = 0; i < sequence.length; i++ )
		{
			var pos  = this.getTilePos( sequence[ i ].tile, tmpWidth );
			var tile = new Surface( '0', _Width, _Width );
			tile.drawImageRect( _SpriteTiles.getCanvas(), pos.x, pos.y, _Width, _Width, 0, 0 );
			_Frameset.push( { Pic : tile, Time : sequence[ i ].time } );
		}
	}

//--------------------------------------------------------------------//

	this.animations = function( animations )
	{
		if( animations )
		{
			_Animation = animations;
		} else {
			return _Animation;
		}
	}

//--------------------------------------------------------------------//

	this.currentAnimation = function( name, loop )
	{
		if( name )
		{
			_CurAnim   = name;
			_FrameIdx  = 0;
			_Tic       = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
			_EOA       = false;
			if( loop )
			{
				_AnimLoop  = true; // Must find a way to work with it
			} else {
				_AnimLoop  = false;
			}
		} else {
			return _CurAnim;
		}
	}
	
//--------------------------------------------------------------------//

	this.initialize = function( params )
	{
		if( params.name       ) this.name = params.name;
		if( params.tilemap    ) this.loadSpriteTiles( params.tilemap.file, params.tilemap.width );
		if( params.frames     ) this.loadFrames( params.frames );
		if( params.animations ) this.animations( params.animations );
		if( params.animation  ) this.currentAnimation( params.animation.name, params.animation.loop );
		if( params.position   ) this.position( params.position );
	}

//--------------------------------------------------------------------//

}
/*
	Clase: world.class.js
	Este objeto debería definir una instancia del mundo de juego
	Se deben pasar 3 parametros:
	* surBackground: Tipo Surface, es la imagen que va en el fondo del parallax,
					   la imagen debe ser de 1536x768
	* surTilemap:    Tipo Surface, imagen que contiene los tiles a usar en el nivel,
	 			       los tiles son de 48x48 sin espacio de separación
	* arrLevelMap:   Tipo Array bidimensional, contiene los números de tile
	 				   a colocar en el mapa
	
*/ 


var World = function( surBackground, surTilemap, arrLevelMap, arrCollitions )
{
	var _layers = [];
	var _surBackground;
	var _surViewport = new Surface('0', 1008, 576 );;
	var _intTilemap;
	
////////////////////////////////////////////////////////////////////////
//           INICIALIZA EL CANVAS Y EL CONTEXTO                       //
////////////////////////////////////////////////////////////////////////	
	this._init = function( surBackground, surTilemap, arrLevelMap, arrCollitions )
	{
		var tile_layer = {}

		Layer_Tiles.call( tile_layer );

		tile_layer.initialize( 
		{
			tilemap      : surTilemap,
			levelmap     : arrLevelMap,
			collitionmap : arrCollitions
		});

		_layers.push( tile_layer );
		
		_surBackground = surBackground;
	}

/* ------------------------------------------------------------------ */	

	this.checkCollition = function( coords, layer )
	{
		if( ! layer ) layer = 0;
		return _layers[ layer ].checkCollition( coords );
	}

/* ------------------------------------------------------------------ */	

	this.expandCollitionMap = function( arrColMap, layer )
	{
		if( ! layer ) layer = 0;
		_layers[ layer ].expandCollitionMap( arrColMap );
	}
	
/* ------------------------------------------------------------------ */	

	this.getViewport = function( intX, intY)
	{
		// Sanity check para mantener el viewport dentro del mundo
		intX = ( intX < 0 ? 0 : ( intX > 2112 ? 2112 : intX ) );
		intY = ( intY < 0 ? 0 : ( intY > 384 ? 384 : intY ) );
		
		_surViewport.drawImageRect( _surBackground.getCanvas(), intX >> 2, intY >> 2, 1008, 576, 0, 0 );
		var intTileX = Math.floor( intX / 48 );
		var intTileY = Math.floor( intY / 48 );
		
		// en principio vamos a hacer un full redraw de cada cuadro
		// una vez que esto ande veré de optimizar esto par no desperdiciar tantos recursos :-)
		_layers[ 0 ].fillTiles( intTileX, intTileY );
		//this.fillTiles( intTileX, intTileY );
		
		intTileX = intX % 48;
		intTileY = intY % 48;
		_surViewport.drawImageRect( _layers[ 0 ].getCanvas(), intTileX, intTileY, 1008, 576, 0, 0 );
		
		return _surViewport;
	}
	
	
////////////////////////////////////////////////////////////////////////
//                     INICIALIZA LA CLASE                            //
////////////////////////////////////////////////////////////////////////

	if( ! surBackground && ! surTilemap && ! arrLevelMap ) 
	{
		console.log('new World(): ERROR - Faltan parámetros de inicialización');
	} else { 
		this._init( surBackground, surTilemap, arrLevelMap, arrCollitions );
	}
}
/*
	Clase: input.class.js
	Esta clase se encarga de interceptar eventos de teclado y mouse
	Si se desean capturar eventos de mouse se debe pasar true al momento
	de inicializar la clase
	
*/

var Input = function( bolMouseCatch ){
	var globalkeys = {};
	var controls;
	var mousepos  = new Array();
	var keyStatus = new Array();
	
// ------------------------------------------------------------------ //	
	
	this.clearBuffer = function()
	{
		for(var i=0; i < globalkeys.controls.length; i++ )
		{
			globalkeys.setKeyState( globalkeys.controls[ i ], 0 );
		}
	}
	
// ------------------------------------------------------------------ //

	globalkeys.setKeyState = function( KeyID, status ){
		switch( KeyID ){
			case this.controls[ 6 ] :
				this.keyStatus.fire2 = status;
				break;
			case this.controls[ 4 ] :
				this.keyStatus.menu = status;
				break;
			case this.controls[ 5 ] :
				this.keyStatus.fire1 = status;
				break;
			case this.controls[ 3 ] :
				this.keyStatus.left = status;
				break;
			case this.controls[ 0 ] :
				this.keyStatus.up = status;
				break;
			case this.controls[ 1 ] :
				this.keyStatus.right = status;
				break;
			case this.controls[ 2 ] :
				this.keyStatus.down = status;
				break;
			case this.controls[ 7 ] :
				this.keyStatus.lclick = status;
				break;
			case this.controls[ 8 ] :
				this.keyStatus.rclick = status;
				break;
			case this.controls[ 9 ] :
				this.keyStatus.mclick = status;
				break;
		}
	}

// ------------------------------------------------------------------ //
	
	this.detectMouseMove = function( e ){
		var evt = e || window.event;
		this.mousepos.x = evt.clientX;
		this.mousepos.y = evt.clientY;	
	}

// ------------------------------------------------------------------ //
	
	this.getMouseKeyCode = function( evt ){
		var mouse_code = 0;
		if ( evt.which ) {
			switch( evt.which ){
				case 1 :
					mouse_code = 100;
					break;
				case 2 :
					mouse_code = 102;
					break;
				case 3 :
					mouse_code = 101;
					break;
			}
		} else if ( evt.button ) {
			switch( evt.button ){
				case 1 :
					mouse_code = 100;
					break;
				case 2 :
					mouse_code = 101;
					break;
				case 3 :
					mouse_code = 102;
					break;
			}
		}
		return mouse_code;
	}

// ------------------------------------------------------------------ //
	
	this.detectMouseDown = function ( e ) {
		var evt = e || window.event;
		this.setKeyState( this.getMouseKeyCode( evt ), 1 );
	}

// ------------------------------------------------------------------ //
	
	this.detectMouseUp = function( e ) {
		var evt = e || window.event;
		this.setKeyState( this.getMouseKeyCode( evt ), 0 );
	}

// ------------------------------------------------------------------ //
	
	this.detectKeyDown = function( e ) {
		var evt = e || window.event;
		KeyID = evt.keyCode;
		globalkeys.setKeyState( KeyID, 1 );
	}

// ------------------------------------------------------------------ //
	
	this.detectKeyUp = function(e) {
		var evt = e || window.event;
		KeyID = evt.keyCode;
		globalkeys.setKeyState( KeyID, 0 );
	}

// ------------------------------------------------------------------ //
	
	this.KeyStatus = function( strKey ){
		return globalkeys.keyStatus[ strKey ];
	}

// ------------------------------------------------------------------ //
	
	this.MousePos = function( strAxis ){
		return this.mousepos[ strAxis ];
	}

// ------------------------------------------------------------------ //
	
	this.MouseCatch = function( bolActivate ){
		if( bolActivate ){
			document[ 'onmousemove' ] = this.detectMouseMove;
			window[ 'onmousedown'   ] = this.detectMouseDown;
			window[ 'onmouseup'     ] = this.detectMouseUp;
		} else {
			document[ 'onmousemove' ] = null;
			window[ 'onmousedown'   ] = null;
			window[ 'onmouseup'     ] = null;
			globalkeys.setKeyState( 100, 0 );
			globalkeys.setKeyState( 101, 0 );
			globalkeys.setKeyState( 102, 0 );
			globalkeys.mousepos.x = 0;
			globalkeys.mousepos.y = 0;
		}
	}

// ------------------------------------------------------------------ //
	
	this._init = function( bolCatchMouse ){
		globalkeys.controls = [
					38,  // Up
					39,  // Right
					40,  // Down
					37,  // Left
					27,  // Escape: Menu
					32,  // Space: Fire1
					17,  // Control: Fire2
					100, // Left Click
					101, // Right Click
					102  // Middle Click
				];
		globalkeys.mousepos = {'x' :0, 'y' :0 };
		globalkeys.keyStatus = {
					'up'    : 0,
					'right' : 0,
					'down'  : 0,
					'left'  : 0,
					'menu'  : 0,
					'fire1' : 0,
					'fire2' : 0,
					'rclick': 0,
					'lclick': 0,
					'mclick': 0
				};	
		
		document[ 'onkeydown' ] = this.detectKeyDown;
		document[ 'onkeyup'   ] = this.detectKeyUp;
		
		this.MouseCatch( bolMouseCatch );
	}

// ------------------------------------------------------------------ //
	
	this._init( bolMouseCatch );
	
}
