/*
	Class: layer_tiles.class.js
	
*/ 


var Layer = function()
{
	this._size     = { width : 1104, height : 672, view_width : 1008, view_height : 576 };
	this._position = { top : 0, left : 0 };
	this._viewpos  = { top : 0, left : 0 };
	this._layer    = null;
	this._tilemap  = null;
	this._viewport = null;
	this._collitionMask = null;
	this._intTilemap    = null;
	
/* ------------------------------------------------------------------ */	

	this.Position = function( position )
	{
		if( position )
		{
			if( position.top && position.left ) this._position = position;
		} else {
			return this._position;
		}
	}

/* ------------------------------------------------------------------ */	

	this.viewPosition = function( viewposition )
	{
		if( viewposition )
		{
			if( ! isNaN( viewposition.top ) &&  ! isNaN( viewposition.left ) ) 
				this._viewpos = viewposition;
		} else {
			return this._viewpos;
		}
	}
		
/* ------------------------------------------------------------------ */	

	this._initialize = function( data )
	{
		if( data.name         ) _name          = data.name;
		if( data.tilemap      ) this._tilemap       = data.tilemap;
        if( data.dimensions   ) this._size          = data.dimensions;
        if( data.position     ) this._position      = data.position;
        if( data.viewposition ) this._viewpos       = data.viewposition;
        if( data.tilemap      ) 
        {
			this.initCollitionMap();			
			if( data.collitionmap ) this._collitionMask = this.expandCollitionMap( data.collitionmap );
		}
		// Define a 23x14 tiles viewport (tiles of 48x48 pixels)
		// 1104 = 23*48
		// 672 = 14*48
		this._intTilemap = ( data.tilemap.Size().width / 48 );
		this._layer      = new Surface('0', this._size.width, this._size.height );
		this._viewport   = new Surface('0', this._size.view_width, this._size.view_height );
		
	}

/* ------------------------------------------------------------------ */	

	this.initCollitionMap = function()
	{
		this._collitionMask = new Array( ( this._tilemap.Size().width / 48 ) * (this._tilemap.Size().height / 48 ) );
		for( var i = 0; i < this._collitionMask.length; i++)
		{
			this._collitionMask[ i ] = new Array( 6 );
			for( var j = 0; j < 6; j++ )
			{
				this._collitionMask[ i ][ j ] = new Array( 6 );
				for( var k = 0; k < 6; k++ )
				{
					this._collitionMask[ i ][ j ][ k ] = 0;
				}
			}
		}
	}
		
/* ------------------------------------------------------------------ */	

	this.expandCollitionMap = function( arrColMap )
	{		
		for(var i=0; i < arrColMap.length; i++)
			for(var j=0; j<6; j++)
			{
				var z = parseInt( arrColMap[ i ].substr( ( j * 2 ), 2 ), 16 );
				var x = 32;
				for(var k = 0; k < 6; k++ )
				{
					this._collitionMask[ i ][ j ][ k ] = ((z & x) > 0 ? 1 : 0 );
					x = x >> 1;
				}
			}
	}
		
/* ------------------------------------------------------------------ */	

	this.getCanvas = function()
	{
		return this._layer.getCanvas();
	}
	
/* ------------------------------------------------------------------ */

	this.getViewport = function()
	{
		var maxwidth  = ( this._size.width  - this._size.view_width  );
		var maxheight = ( this._size.height - this._size.view_height );
		intX = ( this._viewpos.left < 0 ? 0 : ( this._viewpos.left > maxwidth  ? maxwidth  : this._viewpos.left ) );
		intY = ( this._viewpos.top  < 0 ? 0 : ( this._viewpos.top  > maxheight ? maxheight : this._viewpos.top  ) );
			
		intTileX = intX % 48;
		intTileY = intY % 48;
		this._viewport.drawImageRect( this._layer.getCanvas(), intTileX, intTileY, this._size.view_width, this._size.view_height, 0, 0 );

		return this._viewport;
	}
	
}
