	
var Primitives = Surface;

////////////////////////////////////////////////////////////////////////////////////////////////
//                              RUTINAS DE PROCESO DE TEXTOS                                  //
////////////////////////////////////////////////////////////////////////////////////////////////

Primitives.prototype = 
{
	// Prototipo que dibuja un texto en pantalla
	_text : function(font, color, x, y, text, bold, outline)
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.font        = font;
		if( outline == false ) this.ctxSurface.fillText( text, x, y );
		if( bold    == true  ) this.ctxSurface.strokeText( text, x, y );
	},

	// Pone un texto normal o con estilo negrita (por font)
	text : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, false, false );
	},
	
	// pone un texto y dibuja un outline para hacerlo negrita (por canvas)
	// si se pasa bold en el font entonces tenes hipernegrita
	text_bold : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, true, false);
	},

	// Pone un texto en outline
	text_outline : function( font, color, x, y, text )
	{
		this._text( font, color, x, y, text, true, true );
	},

	// Pone un texto normal o con estilo negrita (por font)
	textSize : function( font, text )
	{
		var pa  = pa || document.body;
		var who = document.createElement( 'div' );
		
		who.style.visibility = 'hidden';
		who.style.position   = 'absolute';
		who.style.font       = font;
		who.appendChild( document.createTextNode( text ) );
		
		pa.appendChild( who );
		var fs = [ who.offsetWidth, who.offsetHeight ];
		pa.removeChild( who );
		return fs;
	},
	
////////////////////////////////////////////////////////////////////////////////////////////////
//                                RUTINAS DE RENDER EN CANVAS                                 //
////////////////////////////////////////////////////////////////////////////////////////////////

	// Funcion que devuelve un gradiente lineal de 2 pasos
	gradient : function( x1, y1, x2, y2, color1, color2 )
	{
		var gradient = this.ctxSurface.createLinearGradient( x1, y1, x2, y2 );
		gradient.addColorStop(0, color1);
		gradient.addColorStop(1, color2);
		return gradient;
	},
		
	// Dibuja una linea del color especificado entre los puntos pasados x parametro
	line : function( x1, y1, x2, y2, color )
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.beginPath();
		this.ctxSurface.moveTo( x1, y1 );
		this.ctxSurface.lineTo( x2, y2 );
		this.ctxSurface.stroke();
	},

	// Dibuja una linea con un gradiente
	line_gradient : function( x1, y1, x2, y2, color1, color2 )
	{
		var grad = this.gradient( x1, y1, x2, y2, color1, color2 );
		this.line(x1, y1, x2, y2, grad );
	},

	// Funcion que dibuja un rectangulo del tamaño especificado en el canvas
	// El color puede ser reemplazado por un gradiente
	line_fill : function( x, y, width, height, color ) 
	{
		this.ctxSurface.fillStyle = color;
		this.ctxSurface.fillRect( x, y, width, height );
	},
	
	// Funcion que dibuja un punto del tamaño especificado en el canvas
	pset : function( x, y, side, color ) 
	{
		this.ctxSurface.fillStyle = color;
		this.ctxSurface.fillRect( x, y, side, side );
	},

	// Prototipo de arco que permite dibujar arcos (abiertos o cerrado) y circulos 
	_arc : function( x, y, radio, degini, degfin, color, closed, fill)
	{
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.beginPath();
		radini = this.deg2rad( degini );
		radfin = this.deg2rad( degfin );
		this.ctxSurface.arc( x, y, radio, radini, radfin, true ); 
		if( closed == true ) this.ctxSurface.closePath();
		if( fill   == true ) this.ctxSurface.fill();
		this.ctxSurface.stroke();
	},
	
	circle : function(x, y, radio, color, fill )
	{
		if( ! fill ) fill = false;
		this._arc( x, y, radio, 0, 360, color, true, fill );
	},
	
	// Arco abierto y hueco
	arc : function(x, y, radio, degini, degfin, color )
	{
		this._arc(x, y, radio, degini, degfin, color, false, false );
	},
	
	// Arco cerrado y hueco
	arc_close : function( x, y, radio, degini, degfin, color )
	{
		this._arc( x, y, radio, degini, degfin, color, true, false );
	},
		
	// Arco cerrado y relleno
	arc_fill : function( x, y, radio, degini, degfin, color )
	{
		this._arc( x, y, radio, degini, degfin, color, true, true );
	},
	
	// Rutina de dibujo de poligonos irregulares
	// Esta rutina es el prototipo que permite poligonos
	// abiertos, cerrados, rellenos y con gradientes
	_poly : function( vertex, color, closed, fill )
	{
		this.ctxSurface.strokeStyle = color;
		this.ctxSurface.fillStyle   = color;
		this.ctxSurface.beginPath();
		var start = true;
		for ( var v in vertex ) {
			var k = vertex[v].split(',');
			if(start == true){
				this.ctxSurface.moveTo( k[ 0 ], k[ 1 ] );
			} else {
				this.ctxSurface.lineTo( k[ 0 ], k[ 1 ] );
			}
			start = false;
		}
		if( closed == true ) this.ctxSurface.closePath();
		if( fill   == true ) this.ctxSurface.fill();
		ctx.stroke();
	},
	
	// Acceso simple para poligono abierto
	poly : function( vertex, color )
	{
		this._poly( vertex, color, false, false );
	},
	
	// Acceso simple para poligono cerrado sin relleno
	poly_closed : function(vertex, color )
	{
		this._poly( vertex, color, true, false );
	},
	
	// Acceso simple para poligono 
	poly_fill : function(vertex, color )
	{
		this._poly( vertex, color, true, true );
	},
	
////////////////////////////////////////////////////////////////////////////////////////////////
//                             RUTINAS AUXILIARES INDISPENSABLES                              //
////////////////////////////////////////////////////////////////////////////////////////////////

	// Funcion que pasa de grados sexagecimales a radianes
	deg2rad : function( deg ) 
	{
		return ( deg * Math.PI ) / 180;
	}

};
