var Sprite = function()
{
	
//--------------------------------------------------------------------//
	
	var _Position  = { x: 0, y : 0 };
	var _SpriteTiles;
	var _Frameset  = [];         // Collection of frames format [ { Pic : <Surface>, time : <seconds> } ]
	var _Animation = [];         // Collection of animations format { 'name' : [0,1,2...n], 'name2' : [...] } the array  is a collection of indexes of the _Frameset object
	var _CurAnim   = '';         // Name of the current animation
	var _FrameIdx  = 0;          // Index of the frame in the current animation
	var _AnimLoop  = true;
	var _Width     = 0;
	var _Tic       = 0;
	var _Name      = "Sprite";   // Name of the current sprite
	var _EOA       = false;     // End of Animation

//--------------------------------------------------------------------//

	this.name = function( set )
	{
		if( set )
		{
			_Name = set;
		} else {
			return _Name;
		}
	}
	
//--------------------------------------------------------------------//	

	this.position = function( set )
	{
		if( set )
		{
			if( set.x ) _Position.x = set.x;
			if( set.y ) _Position.y = set.y;
		} else {
			return _Position;
		}
	}

//--------------------------------------------------------------------//
	
	this.eoa = function()
	{
		return _EOA;
	}
	
//--------------------------------------------------------------------//	

	this.tic = function( set )
	{
		if( set )
		{
			_Tic -= set;
			if( _Tic <= 0)
			{
				_FrameIdx++;
				if( _Animation[ _CurAnim ].length == _FrameIdx )
				{
					if( _AnimLoop == false )
					{
						_EOA = true;
						_FrameIdx--;
					} else {
						_FrameIdx = 0;
						_Tic = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
					}
				} else {
					_Tic = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
				}
			}
		} 
	}
	
//--------------------------------------------------------------------//

	this.getFrame = function()
	{
		return _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Pic.getCanvas();
	}
	
//--------------------------------------------------------------------//
	
	this.getTilePos = function( intTileNum, intTileWidth )
	{
		return { 
					x : ( intTileNum * _Width ) % intTileWidth, 
					y : Math.floor( (intTileNum * _Width) / intTileWidth ) * _Width
				};
	}

//--------------------------------------------------------------------//
	
	this.loadSpriteTiles = function( spriteTiles, spriteWidth )
	{
		_SpriteTiles = spriteTiles;
		_Width       = spriteWidth;
	}
	
//--------------------------------------------------------------------//

	this.loadFrames = function( sequence )
	{
		var tmpWidth = _SpriteTiles.Size().width;
		for( var i = 0; i < sequence.length; i++ )
		{
			var pos  = this.getTilePos( sequence[ i ].tile, tmpWidth );
			var tile = new Surface( '0', _Width, _Width );
			tile.drawImageRect( _SpriteTiles.getCanvas(), pos.x, pos.y, _Width, _Width, 0, 0 );
			_Frameset.push( { Pic : tile, Time : sequence[ i ].time } );
		}
	}

//--------------------------------------------------------------------//

	this.animations = function( animations )
	{
		if( animations )
		{
			_Animation = animations;
		} else {
			return _Animation;
		}
	}

//--------------------------------------------------------------------//

	this.currentAnimation = function( name, loop )
	{
		if( name )
		{
			_CurAnim   = name;
			_FrameIdx  = 0;
			_Tic       = _Frameset[ _Animation[ _CurAnim ][ _FrameIdx ] ].Time;
			_EOA       = false;
			if( loop )
			{
				_AnimLoop  = true; // Must find a way to work with it
			} else {
				_AnimLoop  = false;
			}
		} else {
			return _CurAnim;
		}
	}
	
//--------------------------------------------------------------------//

	this.initialize = function( params )
	{
		if( params.name       ) this.name = params.name;
		if( params.tilemap    ) this.loadSpriteTiles( params.tilemap.file, params.tilemap.width );
		if( params.frames     ) this.loadFrames( params.frames );
		if( params.animations ) this.animations( params.animations );
		if( params.animation  ) this.currentAnimation( params.animation.name, params.animation.loop );
		if( params.position   ) this.position( params.position );
	}

//--------------------------------------------------------------------//

}
