/*
	Clase: world.class.js
	Este objeto debería definir una instancia del mundo de juego
	Se deben pasar 3 parametros:
	* surBackground: Tipo Surface, es la imagen que va en el fondo del parallax,
					   la imagen debe ser de 1536x768
	* surTilemap:    Tipo Surface, imagen que contiene los tiles a usar en el nivel,
	 			       los tiles son de 48x48 sin espacio de separación
	* arrLevelMap:   Tipo Array bidimensional, contiene los números de tile
	 				   a colocar en el mapa
	
*/ 


var World = function( surBackground, surTilemap, arrLevelMap, arrCollitions )
{
	var _layers = [];
	var _surBackground;
	var _surViewport = new Surface('0', 1008, 576 );;
	var _intTilemap;
	
////////////////////////////////////////////////////////////////////////
//           INICIALIZA EL CANVAS Y EL CONTEXTO                       //
////////////////////////////////////////////////////////////////////////	
	this._init = function( surBackground, surTilemap, arrLevelMap, arrCollitions )
	{
		var tile_layer = new Layer_Tiles();

		tile_layer.initialize( 
		{
			tilemap      : surTilemap,
			levelmap     : arrLevelMap,
			collitionmap : arrCollitions
		});

		_layers.push( tile_layer );
		
		_surBackground = surBackground;
	}

/* ------------------------------------------------------------------ */	

	this.checkCollition = function( coords, layer )
	{
		if( ! layer ) layer = 0;
		return _layers[ layer ].checkCollition( coords );
	}

/* ------------------------------------------------------------------ */	

	this.expandCollitionMap = function( arrColMap, layer )
	{
		if( ! layer ) layer = 0;
		_layers[ layer ].expandCollitionMap( arrColMap );
	}
	
/* ------------------------------------------------------------------ */	

	this.getViewport = function( intX, intY)
	{
		// Sanity check para mantener el viewport dentro del mundo
		intX = ( intX < 0 ? 0 : ( intX > 2112 ? 2112 : intX ) );
		intY = ( intY < 0 ? 0 : ( intY > 384 ? 384 : intY ) );
		
		_surViewport.drawImageRect( _surBackground.getCanvas(), intX >> 2, intY >> 2, 1008, 576, 0, 0 );
		var intTileX = Math.floor( intX / 48 );
		var intTileY = Math.floor( intY / 48 );
		
		// en principio vamos a hacer un full redraw de cada cuadro
		// una vez que esto ande veré de optimizar esto par no desperdiciar tantos recursos :-)
		_layers[ 0 ].fillTiles( intTileX, intTileY );
		//this.fillTiles( intTileX, intTileY );
		
		intTileX = intX % 48;
		intTileY = intY % 48;
		_surViewport.drawImageRect( _layers[ 0 ].getCanvas(), intTileX, intTileY, 1008, 576, 0, 0 );
		
		return _surViewport;
	}
	
	
////////////////////////////////////////////////////////////////////////
//                     INICIALIZA LA CLASE                            //
////////////////////////////////////////////////////////////////////////

	if( ! surBackground && ! surTilemap && ! arrLevelMap ) 
	{
		console.log('new World(): ERROR - Faltan parámetros de inicialización');
	} else { 
		this._init( surBackground, surTilemap, arrLevelMap, arrCollitions );
	}
}
